/*
 * (C) Copyright 2016
 * Sriram V. <vsriram@mistralsolutions.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <wave.h>

static int do_aplay(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	unsigned  char *addr;
	struct wave *header;
	int ret;

	if (argc < 2 || argc > 2)
		return CMD_RET_USAGE;

	addr = simple_strtoul(argv[1], NULL, 16);

	header = parse_wave(addr);

	if (!header) {
		printf ("Wave file found at %x is NOT valid\n", addr);
		printf ("Pass a valid Wave file and try again\n");
		return 1;
	}

	if ( (header->channels < MONO) || (header->channels > STEREO) ) {
		printf ("This command supports mono/stereo wave files\n");
		return 1;
	}

	if (header->sample_rate != 44100) {
		printf ("The Sample rate %d is NOT supported by this command\n", header->sample_rate);
		printf ("This command currently supports a sampling rate of %d hz ONLY\n", 44100);
		return 1;
	}

	/* skip wave header */
	addr = addr + 44;

	if (sound_init() != 0) {
		printf ("Unable to initialise iis to play audio\n");
		return 1;
	}

	(void) playwave(header, addr);

	return 0;
}

U_BOOT_CMD(
	aplay,	2,	1,	do_aplay,
	"Plays a wave file",
        "<aplay> <address>"
        "\n      - address of the location where the wavefile is stored"
);
