/*
 * (C) Copyright 2016
 * Sriram V. <vsriram@mistralsolutions.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */


#include <common.h>
#include <sound.h>
#include <wave.h>

/*
 * Parses a wavefile, returns valid header on success
 *
 */

static struct wave wheader;
struct wave * parse_wave (unsigned char * wave)
{
	unsigned char *p;
	unsigned char buf[5];

	if (!wave) return NULL;

	memset(&wheader, 0, sizeof(wheader));
	p = wave;

	buf[0] = *p++;
	buf[1] = *p++;
	buf[2] = *p++;
	buf[3] = *p++;
	buf[4] = 0;

	strncpy(wheader.riff, buf, sizeof(wheader.riff));

	if (strncmp (buf, "RIFF", 4) != 0)
		return NULL;

	buf[0] = *p++;
	buf[1] = *p++;
	buf[2] = *p++;
	buf[3] = *p++;

	wheader.chunk_size = buf[3] << 24 | buf[2] << 16 | buf[1] << 8 | buf[0];

	buf[0] = *p++;
	buf[1] = *p++;
	buf[2] = *p++;
	buf[3] = *p++;
	buf[4] = 0;

	strncpy(wheader.wave, buf, sizeof(wheader.wave));

	if (strncmp (buf, "WAVE", 4) != 0)
		return NULL;

	buf[0] = *p++;
	buf[1] = *p++;
	buf[2] = *p++;
	buf[3] = *p++;

	if (strncmp (buf, "fmt ", 4) != 0)
		return NULL;

	strncpy(wheader.fmt_marker, buf, sizeof(wheader.fmt_marker));

	buf[0] = *p++;
	buf[1] = *p++;
	buf[2] = *p++;
	buf[3] = *p++;

	wheader.fmt_len = buf[3] << 24 | buf[2] << 16 | buf[1] << 8 | buf[0];

	buf[0] = *p++;
	buf[1] = *p++;

	wheader.fmt_type = buf[1] << 8 | buf[0];

	buf[0] = *p++;
	buf[1] = *p++;

	wheader.channels = buf[1] << 8 | buf[0];

	buf[0] = *p++;
	buf[1] = *p++;
	buf[2] = *p++;
	buf[3] = *p++;

	wheader.sample_rate = buf[3] << 24 | buf[2] << 16 | buf[1] << 8 | buf[0];

	buf[0] = *p++;
	buf[1] = *p++;
	buf[2] = *p++;
	buf[3] = *p++;

	wheader.byterate = buf[3] << 24 | buf[2] << 16 | buf[1] << 8 | buf[0];

	buf[0] = *p++;
	buf[1] = *p++;

	wheader.block_align = buf[1] << 8 | buf[0];

	buf[0] = *p++;
	buf[1] = *p++;

	wheader.bits_per_sample = buf[1] << 8 | buf[0];

	buf[0] = *p++;
	buf[1] = *p++;
	buf[2] = *p++;
	buf[3] = *p++;
	buf[4] = 0;

	strncpy(wheader.chunk_header, buf, sizeof(wheader.chunk_header));

	buf[0] = *p++;
	buf[1] = *p++;
	buf[2] = *p++;
	buf[3] = *p++;

	wheader.data_size = buf[3] << 24 | buf[2] << 16 | buf[1] << 8 | buf[0];


#if DEBUG_WAVE
	strncpy (buf, wheader.riff,  4);
	buf[4] = 0;
	printf ("riff tag is %s\n", wheader.riff);

	strncpy (buf, wheader.wave,  4);
	buf[4] = 0;
	printf ("wave tag is  %s\n", wheader.wave);

	printf ("overall_size: %d\n", wheader.chunk_size);
	printf ("length_of_fmt: %d\n", wheader.fmt_len);
	printf ("format_type: %d\n", wheader.fmt_type);
	printf ("channels: %d\n", wheader.channels);
	printf ("sample_rate: %d\n", wheader.sample_rate);
	printf ("byterate: %d\n", wheader.byterate);
	printf ("block_align: %d\n", wheader.block_align);
	printf ("bits_per_sample: %d\n", wheader.bits_per_sample);
	printf ("data_size: %d\n", wheader.data_size);

	printf ("num of samples: %d\n", (wheader.data_size * 8)/(wheader.channels * wheader.bits_per_sample));
	printf ("Size of sample: %d\n", wheader.channels * wheader.bits_per_sample/8);
#endif


	return &wheader;
}

int playwave (struct wave *h, char * audio)
{
	int nsamples;
	int ret;

	if (!h || !audio)
		return 0;

	nsamples = (h->data_size * 8)/(h->channels * h->bits_per_sample);

	return play_audio((short *) audio, nsamples, h->channels);
}
